@@ -1,4 +1,5 @@
 __all__ = [
+    "PassThroughOptions",
     "Undefined",
     "UndefinedType",
     "Unsupported",
@@ -8,9 +9,12 @@ __all__ = [
     "deserialization_method",
     "deserialize",
     "deserializer",
+    "identity",
+    "order",
     "properties",
     "schema",
     "schema_ref",
+    "serialization_default",
     "serialization_method",
     "serialize",
     "serialized",
@@ -20,6 +24,7 @@ __all__ = [
     "validator",
 ]
 
+import warnings
 
 from . import (  # noqa: F401
     cache,
@@ -38,19 +43,35 @@ from .conversions import deserializer, serializer
 from .dependencies import dependent_required
 from .deserialization import deserialization_method, deserialize
 from .metadata import properties
+from .ordering import order
 from .schemas import schema
-from .serialization import serialization_method, serialize
+from .serialization import (
+    PassThroughOptions,
+    serialization_default,
+    serialization_method,
+    serialize,
+)
 from .serialization.serialized_methods import serialized
 from .settings import settings
 from .type_names import schema_ref, type_name
 from .types import Undefined, UndefinedType
+from .utils import identity
 from .validation import ValidationError, validator
 from .visitor import Unsupported
 
 try:
-    from . import graphql  # noqa: F401
+    import graphql as _gql
 
-    __all__.append("graphql")
+    if _gql.__version__.startswith("2."):
+        warnings.warn(
+            f"graphql-core version {_gql.__version__} is incompatible with apischema;\n"
+            "GraphQL schema generation is thus not available."
+        )
+    else:
+        from . import graphql  # noqa: F401
+
+        __all__.append("graphql")
+    del _gql
 except ImportError:
     pass
 
@@ -61,6 +82,8 @@ def __getattr__(name):
             "GraphQL feature requires graphql-core library\n"
             "Run `pip install apischema[graphql]` to install it"
         )
+    if name == "skip":
+        warnings.warn("apischema.skip module is deprecated")
     raise AttributeError(f"module {__name__!r} has no attribute {name!r}")
 
 
@@ -68,8 +91,8 @@ def register_default_conversions():
     """Handle standard library + internal types"""
     from . import std_types  # noqa: F401
 
-    deserializer(ValidationError.deserialize)
-    serializer(ValidationError.serialize)
+    deserializer(ValidationError.from_errors)
+    serializer(ValidationError.errors)
 
 
 register_default_conversions()
