@@ -1,27 +1,17 @@
-from inspect import Parameter, isclass, signature
-from typing import (
-    Any,
-    Callable,
-    Dict,
-    Generic,
-    NoReturn,
-    Optional,
-    Tuple,
-    Type,
-    TypeVar,
-    Union,
-    cast,
-)
+from inspect import Parameter, signature
+from typing import Any, Callable, Dict, Generic, Optional, Tuple, Type, cast
 
 from apischema.types import AnyType
-from apischema.typing import get_type_hints, is_new_type, is_type
+from apischema.typing import (
+    get_type_hints,
+    is_annotated,
+    is_literal,
+    is_new_type,
+    is_type,
+    is_union,
+)
 from apischema.utils import get_origin_or_type
 
-try:
-    from apischema.typing import Annotated, Literal
-except ImportError:
-    Annotated, Literal = ..., ...  # type: ignore
-
 Converter = Callable[[Any], Any]
 
 
@@ -42,7 +32,7 @@ def converter_types(
         else:
             parameters = list(signature(converter).parameters.values())
     except ValueError:  # builtin types
-        if target is None and isclass(converter):
+        if target is None and is_type(converter):
             target = cast(Type[Any], converter)
         if source is None:
             raise TypeError("Converter source is unknown") from None
@@ -61,7 +51,7 @@ def converter_types(
         if source is not None and target is not None:
             return source, target
         types = get_type_hints(converter, None, namespace, include_extras=True)
-        if not types and isclass(converter):
+        if not types and is_type(converter):
             types = get_type_hints(
                 converter.__new__, None, namespace, include_extras=True
             ) or get_type_hints(
@@ -73,28 +63,18 @@ def converter_types(
             except KeyError:
                 raise TypeError("converter source is unknown") from None
         if target is None:
-            try:
-                target = types.pop("return")
-            except KeyError:
-                if isclass(converter):
-                    target = cast(Type, converter)
-                else:
+            if is_type(converter):
+                target = cast(Type, converter)
+            else:
+                try:
+                    target = types.pop("return")
+                except KeyError:
                     raise TypeError("converter target is unknown") from None
     return source, target
 
 
-INVALID_CONVERSION_TYPES = {Union, Annotated, Literal, NoReturn}
-
-
 def is_convertible(tp: AnyType) -> bool:
     origin = get_origin_or_type(tp)
     return is_new_type(tp) or (
-        is_type(origin) and origin not in INVALID_CONVERSION_TYPES
+        is_type(origin) and not (is_literal(tp) or is_annotated(tp) or is_union(origin))
     )
-
-
-T = TypeVar("T")
-
-
-def identity(x: T) -> T:
-    return x
