@@ -2,6 +2,7 @@ from dataclasses import dataclass, field as field_, replace
 from enum import Enum
 from functools import wraps
 from inspect import Parameter, iscoroutinefunction
+from itertools import chain
 from typing import (
     Any,
     AsyncIterable,
@@ -27,10 +28,8 @@ import graphql
 
 from apischema import settings
 from apischema.aliases import Aliaser
-from apischema.conversions import identity
 from apischema.conversions.conversions import AnyConversion, DefaultConversion
 from apischema.conversions.visitor import (
-    CachedConversionsVisitor,
     Conv,
     Deserialization,
     DeserializationVisitor,
@@ -46,6 +45,7 @@ from apischema.graphql.resolvers import (
     resolver_parameters,
     resolver_resolve,
 )
+from apischema.json_schema.schema import get_field_schema, get_method_schema, get_schema
 from apischema.metadata.keys import SCHEMA_METADATA
 from apischema.objects import ObjectField
 from apischema.objects.visitor import (
@@ -53,7 +53,9 @@ from apischema.objects.visitor import (
     ObjectVisitor,
     SerializationObjectVisitor,
 )
-from apischema.schemas import Schema, get_schema, merge_schema
+from apischema.ordering import Ordering, sort_by_order
+from apischema.recursion import RecursiveConversionsVisitor
+from apischema.schemas import Schema, merge_schema
 from apischema.serialization import SerializationMethod, serialize
 from apischema.serialization.serialized_methods import ErrorHandler
 from apischema.type_names import TypeName, TypeNameFactory, get_type_name
@@ -61,21 +63,23 @@ from apischema.types import AnyType, NoneType, OrderedDict, Undefined, Undefined
 from apischema.typing import get_args, get_origin, is_annotated
 from apischema.utils import (
     Lazy,
+    as_predicate,
     context_setter,
+    deprecate_kwargs,
     empty_dict,
     get_args2,
     get_origin2,
     get_origin_or_type,
+    identity,
     is_union_of,
-    literal_values,
-    sort_by_annotations_position,
     to_camel_case,
 )
 
-JsonScalar = graphql.GraphQLScalarType(
-    "JSON",
-    specified_by_url="http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf",
-)
+JsonScalar = graphql.GraphQLScalarType("JSON")
+if graphql.version_info >= (3, 1, 2):
+    JsonScalar.specified_by_url = (
+        "http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf"
+    )
 GRAPHQL_PRIMITIVE_TYPES = {
     int: graphql.GraphQLInt,
     float: graphql.GraphQLFloat,
@@ -109,6 +113,16 @@ def exec_thunk(thunk: TypeThunk, *, non_null=None) -> Any:
     return result
 
 
+def get_parameter_schema(
+    func: Callable, parameter: Parameter, field: ObjectField
+) -> Optional[Schema]:
+    from apischema import settings
+
+    return merge_schema(
+        settings.base_schema.parameter(func, parameter, field.alias), field.schema
+    )
+
+
 def merged_schema(
     schema: Optional[Schema], tp: Optional[AnyType]
 ) -> Tuple[Optional[Schema], Mapping[str, Any]]:
@@ -149,29 +163,12 @@ def get_deprecated(
 
 @dataclass(frozen=True)
 class ResolverField:
-    alias: str
     resolver: Resolver
     types: Mapping[str, AnyType]
     parameters: Sequence[Parameter]
     metadata: Mapping[str, Mapping]
     subscribe: Optional[Callable] = None
 
-    @property
-    def name(self) -> str:
-        return self.resolver.func.__name__
-
-    @property
-    def type(self) -> AnyType:
-        return self.types["return"]
-
-    @property
-    def description(self) -> Optional[str]:
-        return get_description(self.resolver.schema)
-
-    @property
-    def deprecated(self) -> Optional[str]:
-        return get_deprecated(self.resolver.schema)
-
 
 IdPredicate = Callable[[AnyType], bool]
 UnionNameFactory = Callable[[Sequence[str]], str]
@@ -231,6 +228,9 @@ def cache_type(method: Method) -> Method:
         ) -> graphql.GraphQLNonNull:
             if name is None:
                 return graphql.GraphQLNonNull(factory.factory(name, description))  # type: ignore
+            # Method is in cache key because scalar types will have the same method,
+            # and then be shared by both visitors, while input/output types will have
+            # their own cache entry.
             if (name, method, description) in self._cache_by_name:
                 tp, cached_args = self._cache_by_name[(name, method, description)]
                 if cached_args == (args, kwargs):
@@ -246,7 +246,7 @@ def cache_type(method: Method) -> Method:
 
 
 class SchemaBuilder(
-    CachedConversionsVisitor[Conv, TypeFactory[GraphQLTp]],
+    RecursiveConversionsVisitor[Conv, TypeFactory[GraphQLTp]],
     ObjectVisitor[TypeFactory[GraphQLTp]],
 ):
     types: Tuple[Type[graphql.GraphQLType], ...]
@@ -254,19 +254,24 @@ class SchemaBuilder(
     def __init__(
         self,
         aliaser: Aliaser,
+        enum_aliaser: Aliaser,
+        enum_schemas: Mapping[Enum, Schema],
         default_conversion: DefaultConversion,
         id_type: graphql.GraphQLScalarType,
         is_id: Optional[IdPredicate],
     ):
         super().__init__(default_conversion)
         self.aliaser = aliaser
+        self.enum_aliaser = enum_aliaser
+        self.enum_schemas = enum_schemas
         self.id_type = id_type
         self.is_id = is_id or (lambda t: False)
         self._cache_by_name: Dict[
-            Tuple[str, Callable, Optional[str]], Tuple[GraphQLTp, Tuple[tuple, dict]]
+            Tuple[str, Callable, Optional[str]],
+            Tuple[graphql.GraphQLNonNull, Tuple[tuple, dict]],
         ] = {}
 
-    def _cache_result(
+    def _recursive_result(
         self, lazy: Lazy[TypeFactory[GraphQLTp]]
     ) -> TypeFactory[GraphQLTp]:
         def factory(name: Optional[str], description: Optional[str]) -> GraphQLTp:
@@ -311,32 +316,43 @@ class SchemaBuilder(
     ) -> TypeFactory[GraphQLTp]:
         return TypeFactory(lambda *_: graphql.GraphQLList(self.visit(value_type).type))
 
-    def _visit_flattened(self, field: ObjectField) -> TypeFactory[GraphQLTp]:
-        raise NotImplementedError
-
     @cache_type
-    def _literal(self, values: Sequence[Any], tp: AnyType) -> TypeFactory[GraphQLTp]:
-        if not all(isinstance(v, str) for v in literal_values(values)):
-            raise TypeError("apischema GraphQL only support Enum/Literal of strings")
-
+    def enum(self, cls: Type[Enum]) -> TypeFactory[GraphQLTp]:
         def factory(
             name: Optional[str], description: Optional[str]
         ) -> graphql.GraphQLEnumType:
             return graphql.GraphQLEnumType(
-                unwrap_name(name, tp),
-                dict(zip(values, values)),
+                unwrap_name(name, cls),
+                {
+                    self.enum_aliaser(name): graphql.GraphQLEnumValue(
+                        member,
+                        get_description(self.enum_schemas.get(member)),
+                        get_deprecated(self.enum_schemas.get(member)),
+                    )
+                    for name, member in cls.__members__.items()
+                },
                 description=description,
             )
 
         return TypeFactory(factory)
 
-    def enum(self, cls: Type[Enum]) -> TypeFactory[GraphQLTp]:
-        return self._literal([elt.value for elt in cls], cls)
-
+    @cache_type
     def literal(self, values: Sequence[Any]) -> TypeFactory[GraphQLTp]:
         from apischema.typing import Literal
 
-        return self._literal(values, Literal[tuple(values)])  # type: ignore
+        if not all(isinstance(v, str) for v in values):
+            raise TypeError("apischema GraphQL only support Literal of strings")
+
+        def factory(
+            name: Optional[str], description: Optional[str]
+        ) -> graphql.GraphQLEnumType:
+            return graphql.GraphQLEnumType(
+                unwrap_name(name, Literal[tuple(values)]),  # type: ignore
+                dict(zip(map(self.enum_aliaser, values), values)),
+                description=description,
+            )
+
+        return TypeFactory(factory)
 
     @cache_type
     def mapping(
@@ -374,18 +390,14 @@ class SchemaBuilder(
         raise TypeError("Tuple are not supported")
 
     def union(self, alternatives: Sequence[AnyType]) -> TypeFactory[GraphQLTp]:
-        factories = [
-            self.visit(alt)
-            for alt in alternatives
-            if alt not in (NoneType, UndefinedType)
-        ]
-        if not factories:
-            raise TypeError("Empty union")
+        factories = self._union_results(
+            (alt for alt in alternatives if alt is not NoneType)
+        )
         if len(factories) == 1:
             factory = factories[0]
         else:
-            factory = self._union_result(factories)
-        if len(factories) != len(alternatives):
+            factory = self._visited_union(factories)
+        if NoneType in alternatives or UndefinedType in alternatives:
 
             def nullable(name: Optional[str], description: Optional[str]) -> GraphQLTp:
                 res = factory.factory(name, description)  # type: ignore
@@ -415,30 +427,61 @@ class SchemaBuilder(
 FieldType = TypeVar("FieldType", graphql.GraphQLInputField, graphql.GraphQLField)
 
 
-def merge_fields(
-    cls: Type,
-    fields: Mapping[str, Lazy[FieldType]],
-    flattened_types: Mapping[str, TypeFactory],
-) -> Dict[str, FieldType]:
-    all_flattened_fields: Dict[str, FieldType] = {}
-    for flattened_name, flattened_factory in flattened_types.items():
-        flattened_type = flattened_factory.raw_type
+class BaseField(Generic[FieldType]):
+    name: str
+    ordering: Optional[Ordering]
+
+    def items(self) -> Iterable[Tuple[str, FieldType]]:
+        raise NotImplementedError
+
+
+@dataclass
+class NormalField(BaseField[FieldType]):
+    alias: str
+    name: str
+    field: Lazy[FieldType]
+    ordering: Optional[Ordering]
+
+    def items(self) -> Iterable[Tuple[str, FieldType]]:
+        yield self.alias, self.field()
+
+
+@dataclass
+class FlattenedField(BaseField[FieldType]):
+    name: str
+    ordering: Optional[Ordering]
+    type: TypeFactory
+
+    def items(self) -> Iterable[Tuple[str, FieldType]]:
+        tp = self.type.raw_type
         if not isinstance(
-            flattened_type,
+            tp,
             (
                 graphql.GraphQLObjectType,
                 graphql.GraphQLInterfaceType,
                 graphql.GraphQLInputObjectType,
             ),
         ):
-            raise TypeError(
-                f"Flattened field {cls.__name__}.{flattened_name} must have an object type"
-            )
-        flattened_fields: Mapping[str, FieldType] = flattened_type.fields
-        if flattened_fields.keys() & all_flattened_fields.keys() & fields.keys():
-            raise TypeError(f"Conflict in flattened fields of {cls}")
-        all_flattened_fields.update(flattened_fields)
-    return {**{name: field() for name, field in fields.items()}, **all_flattened_fields}
+            raise FlattenedError(self)
+        yield from tp.fields.items()
+
+
+class FlattenedError(Exception):
+    def __init__(self, field: FlattenedField):
+        self.field = field
+
+
+def merge_fields(cls: type, fields: Sequence[BaseField]) -> Dict[str, FieldType]:
+    try:
+        sorted_fields = sort_by_order(
+            cls, fields, lambda f: f.name, lambda f: f.ordering
+        )
+    except FlattenedError as err:
+        raise TypeError(
+            f"Flattened field {cls.__name__}.{err.field.name}"
+            f" must have an object type"
+        )
+    return OrderedDict(chain.from_iterable(map(lambda f: f.items(), sorted_fields)))
 
 
 class InputSchemaBuilder(
@@ -448,12 +491,9 @@ class InputSchemaBuilder(
 ):
     types = graphql.type.definition.graphql_input_types
 
-    def _visit_flattened(
-        self, field: ObjectField
-    ) -> TypeFactory[graphql.GraphQLInputType]:
-        return self.visit_with_conv(field.type, field.deserialization)
-
-    def _field(self, field: ObjectField) -> Lazy[graphql.GraphQLInputField]:
+    def _field(
+        self, tp: AnyType, field: ObjectField
+    ) -> Lazy[graphql.GraphQLInputField]:
         field_type = field.type
         field_default = graphql.Undefined if field.required else field.get_default()
         default: Any = graphql.Undefined
@@ -474,20 +514,30 @@ class InputSchemaBuilder(
         return lambda: graphql.GraphQLInputField(
             factory.type,  # type: ignore
             default_value=default,
-            description=get_description(field.schema, field.type),
-            extensions={field.name: ""},
+            description=get_description(get_field_schema(tp, field), field.type),
         )
 
     @cache_type
     def object(
         self, tp: AnyType, fields: Sequence[ObjectField]
     ) -> TypeFactory[graphql.GraphQLInputType]:
-        visited_fields = {
-            self.aliaser(f.alias): self._field(f) for f in fields if not f.is_aggregate
-        }
-        flattened_types = {
-            f.name: self._visit_flattened(f) for f in fields if f.flattened
-        }
+        visited_fields: List[BaseField] = []
+        for field in fields:
+            if not field.is_aggregate:
+                normal_field = NormalField(
+                    self.aliaser(field.alias),
+                    field.name,
+                    self._field(tp, field),
+                    field.ordering,
+                )
+                visited_fields.append(normal_field)
+            elif field.flattened:
+                flattened_fields = FlattenedField(
+                    field.name,
+                    field.ordering,
+                    self.visit_with_conv(field.type, field.deserialization),
+                )
+                visited_fields.append(flattened_fields)
 
         def factory(
             name: Optional[str], description: Optional[str]
@@ -497,21 +547,19 @@ class InputSchemaBuilder(
                 name += "Input"
             return graphql.GraphQLInputObjectType(
                 name,
-                lambda: merge_fields(tp, visited_fields, flattened_types),
+                lambda: merge_fields(get_origin_or_type(tp), visited_fields),
                 description,
             )
 
         return TypeFactory(factory)
 
-    def _union_result(
-        self, results: Iterable[TypeFactory]
+    def _visited_union(
+        self, results: Sequence[TypeFactory]
     ) -> TypeFactory[graphql.GraphQLInputType]:
-        results = list(results)
         # Check must be done here too because _union_result is used by visit_conversion
-        if len(results) == 1:
-            return results[0]
-        else:
+        if len(results) != 1:
             raise TypeError("Union are not supported for input")
+        return results[0]
 
 
 Func = TypeVar("Func", bound=Callable)
@@ -527,25 +575,34 @@ class OutputSchemaBuilder(
     def __init__(
         self,
         aliaser: Aliaser,
+        enum_aliaser: Aliaser,
+        enum_schemas: Mapping[Enum, Schema],
         default_conversion: DefaultConversion,
         id_type: graphql.GraphQLScalarType,
         is_id: Optional[IdPredicate],
         union_name_factory: UnionNameFactory,
         default_deserialization: DefaultConversion,
     ):
-        super().__init__(aliaser, default_conversion, id_type, is_id)
+        super().__init__(
+            aliaser, enum_aliaser, enum_schemas, default_conversion, id_type, is_id
+        )
         self.union_name_factory = union_name_factory
         self.input_builder = InputSchemaBuilder(
-            self.aliaser, default_deserialization, self.id_type, self.is_id
+            self.aliaser,
+            self.enum_aliaser,
+            self.enum_schemas,
+            default_deserialization,
+            self.id_type,
+            self.is_id,
         )
-        # Share the same cache for input_builder in order to share
+        # Share the same cache for input_builder in order to share scalar types
         self.input_builder._cache_by_name = self._cache_by_name
         self.get_flattened: Optional[Callable[[Any], Any]] = None
 
     def _field_serialization_method(self, field: ObjectField) -> SerializationMethod:
         return partial_serialization_method_factory(
             self.aliaser, field.serialization, self.default_conversion
-        )(field.type)
+        )(Optional[field.type] if field.none_as_undefined else field.type)
 
     def _wrap_resolve(self, resolve: Func) -> Func:
         if self.get_flattened is None:
@@ -558,7 +615,7 @@ class OutputSchemaBuilder(
 
             return cast(Func, resolve_wrapper)
 
-    def _field(self, field: ObjectField) -> Lazy[graphql.GraphQLField]:
+    def _field(self, tp: AnyType, field: ObjectField) -> Lazy[graphql.GraphQLField]:
         field_name = field.name
         partial_serialize = self._field_serialization_method(field)
 
@@ -567,15 +624,18 @@ class OutputSchemaBuilder(
             return partial_serialize(getattr(obj, field_name))
 
         factory = self.visit_with_conv(field.type, field.serialization)
+        field_schema = get_field_schema(tp, field)
         return lambda: graphql.GraphQLField(
             factory.type,
             None,
             resolve,
-            description=get_description(field.schema, field.type),
-            deprecation_reason=get_deprecated(field.schema, field.type),
+            description=get_description(field_schema, field.type),
+            deprecation_reason=get_deprecated(field_schema, field.type),
         )
 
-    def _resolver(self, field: ResolverField) -> Lazy[graphql.GraphQLField]:
+    def _resolver(
+        self, tp: AnyType, field: ResolverField
+    ) -> Lazy[graphql.GraphQLField]:
         resolve = self._wrap_resolve(
             resolver_resolve(
                 field.resolver,
@@ -621,7 +681,10 @@ class OutputSchemaBuilder(
                 arg_factory = self.input_builder.visit_with_conv(
                     param_type, param_field.deserialization
                 )
-                description = get_description(param_field.schema, param_field.type)
+                description = get_description(
+                    get_parameter_schema(field.resolver.func, param, param_field),
+                    param_field.type,
+                )
 
                 def arg_thunk(
                     arg_factory=arg_factory, default=default, description=description
@@ -631,14 +694,15 @@ class OutputSchemaBuilder(
                     )
 
                 args[self.aliaser(param_field.alias)] = arg_thunk
-        factory = self.visit_with_conv(field.type, field.resolver.conversion)
+        factory = self.visit_with_conv(field.types["return"], field.resolver.conversion)
+        field_schema = get_method_schema(tp, field.resolver)
         return lambda: graphql.GraphQLField(
             factory.type,  # type: ignore
             {name: arg() for name, arg in args.items()} if args else None,
             resolve,
             field.subscribe,
-            field.description,
-            field.deprecated,
+            get_description(field_schema),
+            get_deprecated(field_schema),
         )
 
     def _visit_flattened(
@@ -653,8 +717,8 @@ class OutputSchemaBuilder(
         def get_flattened(obj):
             return partial_serialize(getattr(get_prev_flattened(obj), field_name))
 
-        with context_setter(self) as setter:
-            setter.get_flattened = get_flattened
+        with context_setter(self):
+            self.get_flattened = get_flattened
             return self.visit_with_conv(field.type, field.serialization)
 
     @cache_type
@@ -665,48 +729,56 @@ class OutputSchemaBuilder(
         resolvers: Sequence[ResolverField] = (),
     ) -> TypeFactory[graphql.GraphQLOutputType]:
         cls = get_origin_or_type(tp)
-        all_fields = {f.alias: self._field(f) for f in fields if not f.is_aggregate}
-        name_by_aliases = {f.alias: f.name for f in fields}
-        all_fields.update({r.alias: self._resolver(r) for r in resolvers})
-        name_by_aliases.update({r.alias: r.resolver.func.__name__ for r in resolvers})
-        for alias, (resolver, types) in get_resolvers(tp).items():
+        visited_fields: List[BaseField[graphql.GraphQLField]] = []
+        flattened_factories = []
+        for field in fields:
+            if not field.is_aggregate:
+                normal_field = NormalField(
+                    self.aliaser(field.name),
+                    field.name,
+                    self._field(tp, field),
+                    field.ordering,
+                )
+                visited_fields.append(normal_field)
+            elif field.flattened:
+                flattened_factory = self._visit_flattened(field)
+                flattened_factories.append(flattened_factory)
+                visited_fields.append(
+                    FlattenedField(field.name, field.ordering, flattened_factory)
+                )
+        resolvers = list(resolvers)
+        for resolver, types in get_resolvers(tp):
             resolver_field = ResolverField(
-                alias,
-                resolver,
-                types,
-                resolver.parameters,
-                resolver.parameters_metadata,
+                resolver, types, resolver.parameters, resolver.parameters_metadata
             )
-            all_fields[alias] = self._resolver(resolver_field)
-            name_by_aliases[alias] = resolver.func.__name__
-        sorted_fields = sort_by_annotations_position(
-            cls, all_fields, name_by_aliases.__getitem__
-        )
-        visited_fields = OrderedDict(
-            (self.aliaser(a), all_fields[a]) for a in sorted_fields
-        )
-        flattened_types = {
-            f.name: self._visit_flattened(f) for f in fields if f.flattened
-        }
+            resolvers.append(resolver_field)
+        for resolver_field in resolvers:
+            normal_field = NormalField(
+                self.aliaser(resolver_field.resolver.alias),
+                resolver_field.resolver.func.__name__,
+                self._resolver(tp, resolver_field),
+                resolver_field.resolver.ordering,
+            )
+            visited_fields.append(normal_field)
 
-        def field_thunk() -> graphql.GraphQLFieldMap:
-            return merge_fields(cls, visited_fields, flattened_types)
-
-        interfaces = list(map(self.visit, get_interfaces(cls)))
         interface_thunk = None
-        if interfaces:
+        interfaces = list(map(self.visit, get_interfaces(cls)))
+        if interfaces or flattened_factories:
 
             def interface_thunk() -> Collection[graphql.GraphQLInterfaceType]:
-                result = {
+                all_interfaces = {
                     cast(graphql.GraphQLInterfaceType, i.raw_type) for i in interfaces
                 }
-                for flattened_factory in flattened_types.values():
+                for flattened_factory in flattened_factories:
                     flattened = cast(
                         Union[graphql.GraphQLObjectType, graphql.GraphQLInterfaceType],
                         flattened_factory.raw_type,
                     )
-                    result.update(flattened.interfaces)
-                return sorted(result, key=lambda i: i.name)
+                    if isinstance(flattened, graphql.GraphQLObjectType):
+                        all_interfaces.update(flattened.interfaces)
+                    elif isinstance(flattened, graphql.GraphQLInterfaceType):
+                        all_interfaces.add(flattened)
+                return sorted(all_interfaces, key=lambda i: i.name)
 
         def factory(
             name: Optional[str], description: Optional[str]
@@ -714,12 +786,15 @@ class OutputSchemaBuilder(
             name = unwrap_name(name, cls)
             if is_interface(cls):
                 return graphql.GraphQLInterfaceType(
-                    name, field_thunk, interface_thunk, description=description
+                    name,
+                    lambda: merge_fields(cls, visited_fields),
+                    interface_thunk,
+                    description=description,
                 )
             else:
                 return graphql.GraphQLObjectType(
                     name,
-                    field_thunk,
+                    lambda: merge_fields(cls, visited_fields),
                     interface_thunk,
                     is_type_of=lambda obj, _: isinstance(obj, cls),
                     description=description,
@@ -732,23 +807,17 @@ class OutputSchemaBuilder(
     ) -> TypeFactory[graphql.GraphQLOutputType]:
         raise TypeError("TypedDict are not supported in output schema")
 
-    def _union_result(
-        self, factories: Iterable[TypeFactory]
+    @cache_type
+    def _visited_union(
+        self, results: Sequence[TypeFactory]
     ) -> TypeFactory[graphql.GraphQLOutputType]:
-        factories = list(factories)  # Execute the iteration (tuple to be hashable)
-
         def factory(
             name: Optional[str], description: Optional[str]
         ) -> graphql.GraphQLOutputType:
-            types = [factory.raw_type for factory in factories]
+            types = [factory.raw_type for factory in results]
             if name is None:
                 name = self.union_name_factory([t.name for t in types])
-            cache_key = (name, self._union_result, description)
-            if cache_key not in self._cache_by_name:
-                self._cache_by_name[cache_key] = graphql.GraphQLNonNull(
-                    graphql.GraphQLUnionType(name, types, description=description)
-                ), ((), {})
-            return self._cache_by_name[cache_key][0]
+            return graphql.GraphQLUnionType(name, types, description=description)
 
         return TypeFactory(factory)
 
@@ -763,8 +832,9 @@ class Operation(Generic[T]):
     function: Callable[..., T]
     alias: Optional[str] = None
     conversion: Optional[AnyConversion] = None
-    schema: Optional[Schema] = None
     error_handler: ErrorHandler = Undefined
+    order: Optional[Ordering] = None
+    schema: Optional[Schema] = None
     parameters_metadata: Mapping[str, Mapping] = field_(default_factory=dict)
 
 
@@ -784,9 +854,7 @@ class Subscription(Operation[AsyncIterable]):
 Op = TypeVar("Op", bound=Operation)
 
 
-def operation_resolver(
-    operation: Union[Callable, Op], op_class: Type[Op]
-) -> Tuple[str, Resolver]:
+def operation_resolver(operation: Union[Callable, Op], op_class: Type[Op]) -> Resolver:
     if not isinstance(operation, op_class):
         operation = op_class(operation)  # type: ignore
     error_handler: Optional[Callable]
@@ -810,16 +878,19 @@ def operation_resolver(
     wrapper.__annotations__ = op.__annotations__
 
     (*parameters,) = resolver_parameters(operation.function, check_first=True)
-    return operation.alias or operation.function.__name__, Resolver(
+    return Resolver(
         wrapper,
+        operation.alias or operation.function.__name__,
         operation.conversion,
-        operation.schema,
         error_handler,
+        operation.order,
+        operation.schema,
         parameters,
         operation.parameters_metadata,
     )
 
 
+@deprecate_kwargs({"union_ref": "union_name"})
 def graphql_schema(
     *,
     query: Iterable[Union[Callable, Query]] = (),
@@ -830,18 +901,20 @@ def graphql_schema(
     description: Optional[str] = None,
     extensions: Optional[Dict[str, Any]] = None,
     aliaser: Optional[Aliaser] = to_camel_case,
-    id_types: Union[Collection[AnyType], IdPredicate] = None,
+    enum_aliaser: Optional[Aliaser] = str.upper,
+    enum_schemas: Optional[Mapping[Enum, Schema]] = None,
+    id_types: Union[Collection[AnyType], IdPredicate] = (),
     id_encoding: Tuple[
         Optional[Callable[[str], Any]], Optional[Callable[[Any], str]]
     ] = (None, None),
-    # TODO deprecate union_ref parameter
-    union_ref: UnionNameFactory = "Or".join,
     union_name: UnionNameFactory = "Or".join,
     default_deserialization: DefaultConversion = None,
     default_serialization: DefaultConversion = None,
 ) -> graphql.GraphQLSchema:
     if aliaser is None:
         aliaser = settings.aliaser
+    if enum_aliaser is None:
+        enum_aliaser = lambda s: s
     if default_deserialization is None:
         default_deserialization = settings.deserialization.default_conversion
     if default_serialization is None:
@@ -854,9 +927,8 @@ def graphql_schema(
         (mutation, Mutation, mutation_fields),
     ]:
         for operation in operations:  # type: ignore
-            alias, resolver = operation_resolver(operation, op_class)
+            resolver = operation_resolver(operation, op_class)
             resolver_field = ResolverField(
-                alias,
                 resolver,
                 resolver.types(),
                 resolver.parameters,
@@ -868,14 +940,15 @@ def graphql_schema(
             sub_op = Subscription(sub_op)  # type: ignore
         sub_parameters: Sequence[Parameter]
         if sub_op.resolver is not None:
-            alias = sub_op.alias or sub_op.resolver.__name__
-            _, subscriber2 = operation_resolver(sub_op, Subscription)
+            subscriber2 = operation_resolver(sub_op, Subscription)
             _, *sub_parameters = resolver_parameters(sub_op.resolver, check_first=False)
             resolver = Resolver(
                 sub_op.resolver,
+                sub_op.alias or sub_op.resolver.__name__,
                 sub_op.conversion,
-                sub_op.schema,
                 subscriber2.error_handler,
+                sub_op.order,
+                sub_op.schema,
                 sub_parameters,
                 sub_op.parameters_metadata,
             )
@@ -890,12 +963,14 @@ def graphql_schema(
                 serialized=False,
             )
         else:
-            alias, subscriber2 = operation_resolver(sub_op, Subscription)
+            subscriber2 = operation_resolver(sub_op, Subscription)
             resolver = Resolver(
                 lambda _: _,
+                subscriber2.alias,
                 sub_op.conversion,
-                sub_op.schema,
                 subscriber2.error_handler,
+                sub_op.order,
+                sub_op.schema,
                 (),
                 {},
             )
@@ -918,16 +993,11 @@ def graphql_schema(
             sub_types = {**sub_types, "return": resolver.return_type(event_type)}
 
         resolver_field = ResolverField(
-            alias,
-            resolver,
-            sub_types,
-            sub_parameters,
-            sub_op.parameters_metadata,
-            subscribe,
+            resolver, sub_types, sub_parameters, sub_op.parameters_metadata, subscribe
         )
         subscription_fields.append(resolver_field)
 
-    is_id = id_types.__contains__ if isinstance(id_types, Collection) else id_types
+    is_id = as_predicate(id_types)
     if id_encoding == (None, None):
         id_type: graphql.GraphQLScalarType = graphql.GraphQLID
     else:
@@ -942,10 +1012,12 @@ def graphql_schema(
 
     output_builder = OutputSchemaBuilder(
         aliaser,
+        enum_aliaser,
+        enum_schemas or {},
         default_serialization,
         id_type,
         is_id,
-        union_name or union_ref,
+        union_name,
         default_deserialization,
     )
 
