@@ -31,26 +31,34 @@ class JsonType(str, Enum):
 
     @staticmethod
     def from_type(cls: Type) -> "JsonType":
-        try:
-            return {
-                NoneType: JsonType.NULL,
-                bool: JsonType.BOOLEAN,
-                str: JsonType.STRING,
-                int: JsonType.INTEGER,
-                float: JsonType.NUMBER,
-                list: JsonType.ARRAY,
-                dict: JsonType.OBJECT,
-            }[cls]
-        except KeyError:
-            raise TypeError(f"Invalid JSON type {cls}")
+        return TYPE_TO_JSON_TYPE[cls]
 
 
-def bad_type(data: Any, expected: type) -> ValidationError:
-    msg = (
-        f"expected type {JsonType.from_type(expected)},"
+class JsonTypes(Dict[type, JsonType]):
+    def __missing__(self, key):
+        raise TypeError(f"Invalid JSON type {key}")
+
+
+TYPE_TO_JSON_TYPE = JsonTypes(
+    {
+        NoneType: JsonType.NULL,
+        bool: JsonType.BOOLEAN,
+        str: JsonType.STRING,
+        int: JsonType.INTEGER,
+        float: JsonType.NUMBER,
+        list: JsonType.ARRAY,
+        dict: JsonType.OBJECT,
+    }
+)
+
+
+def bad_type(data: Any, *expected: type) -> ValidationError:
+    msgs = [
+        f"expected type {JsonType.from_type(tp)},"
         f" found {JsonType.from_type(data.__class__)}"
-    )
-    return ValidationError([msg])
+        for tp in expected
+    ]
+    return ValidationError(msgs)
 
 
 class JsonSchema(Dict[str, Any]):
@@ -96,7 +104,7 @@ def json_schema(
     exclusiveMinimum: Number = None,
     examples: Sequence[Any] = None,
     format: str = None,
-    items: Union[JsonSchema, Sequence[JsonSchema]] = JsonSchema(),
+    items: Union[bool, JsonSchema] = JsonSchema(),
     maximum: Number = None,
     minimum: Number = None,
     maxItems: int = None,
@@ -109,6 +117,7 @@ def json_schema(
     oneOf: Sequence[JsonSchema] = [],
     pattern: Pattern = None,
     patternProperties: Mapping[Pattern, JsonSchema] = {},
+    prefixItems: Sequence[JsonSchema] = [],
     properties: Mapping[str, JsonSchema] = {},
     readOnly: bool = False,
     required: Sequence[str] = [],
